@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #
 #
 # The developers of the Exaile media player hereby grant permission
@@ -30,7 +30,9 @@ import time
 import pygst
 pygst.require('0.10')
 import gst
+import glib
 
+from xl.nls import gettext as _
 from xl import event, settings, common
 from xl.player import pipe
 logger = logging.getLogger(__name__)
@@ -44,7 +46,8 @@ class ExailePlayer(object):
         self.queue = None
         self._name = name
         self._playtime_stamp = None
-        self._last_position = 0
+        self._delay_id = None
+        self._stop_id = None
 
         self._mainbin = pipe.MainBin(self, pre_elems=pre_elems)
         self._pipe = None
@@ -60,7 +63,10 @@ class ExailePlayer(object):
     def _on_option_set(self, name, object, data):
         if data == "%s/volume" % self._name:
             self._load_volume()
-
+        elif data == '%s/audiosink_device' % self._name or \
+             data == '%s/audiosink' % self._name:
+            self._mainbin.setup_audiosink()
+        
     def _on_track_end(self, name, obj, track):
         if not track:
             return
@@ -117,8 +123,12 @@ class ExailePlayer(object):
         elif message.type == gst.MESSAGE_ERROR:
             logger.error("%s %s" %(message, dir(message)) )
             message_text = message.parse_error()[1]
-            # The most readable part is always the last
+            # The most readable part is always the last..
             message_text = message_text[message_text.rfind(':') + 1:]
+            # .. unless there's nothing in it.
+            if ' ' not in message_text:
+                if message_text.startswith('playsink'):
+                    message_text += _(': Possible audio device error, is it plugged in?')
             event.log_event('playback_error', self, message_text)
             self._error_func()
         return True
@@ -127,7 +137,7 @@ class ExailePlayer(object):
         pass # for overriding
 
     def _eos_func(self):
-        logger.warning("Unhandled EOS message: ", message)
+        logger.warning("Unhandled EOS message!")
 
     def _error_func(self):
         self.stop()
@@ -187,6 +197,56 @@ class ExailePlayer(object):
         return self._get_current()
     current = property(__get_current)
 
+    def _cancel_delayed_start(self):
+        if self._delay_id is not None:
+            glib.source_remove(self._delay_id)
+            self._delay_id = None
+    
+    def _should_delay_start(self):
+        delay = settings.get_option('%s/auto_advance_delay' % self._name, 0)
+        if delay <= 0:
+            return False
+        self.pause()
+        self._delay_id = glib.timeout_add(int(delay), self._unpause) 
+        return True
+    
+    def _cancel_stop_offset(self):
+        if self._stop_id is not None:
+            glib.source_remove(self._stop_id)
+            self._stop_id = None
+    
+    def _setup_startstop_offsets(self, track):
+        
+        start_offset = track.get_tag_raw('__startoffset')
+        stop_offset = track.get_tag_raw('__stopoffset')
+        
+        if start_offset > 0:
+            
+            # wait up to 1s for the state to switch, else this fails
+            if self._pipe.get_state(timeout=1000*gst.MSECOND)[0] != gst.STATE_CHANGE_SUCCESS:
+                event.log_event('playback_error', self, "Could not start at specified offset")
+                self._error_func()
+                return
+            
+            self.seek(start_offset)
+            
+        # there's probably a better way to implement this... 
+        if stop_offset > 0:
+            self._stop_id = glib.timeout_add(250, self._monitor_for_stop, track, stop_offset)
+    
+            
+    def _monitor_for_stop(self, track, stop_offset):
+        
+        if track == self.current and self.get_position() >= stop_offset * gst.SECOND and self.is_playing():
+            
+            # send eos to pipe
+            self._pipe.send_event(gst.event_new_eos())
+        
+            self._stop_id = None
+            return False
+        
+        return True
+    
     def play(self, track, **kwargs):
         """
             Starts the playback with the provided track
@@ -214,6 +274,9 @@ class ExailePlayer(object):
                 * `playback_player_end`: indicates the end of playback overall
                 * `playback_track_end`: indicates playback end of a track
         """
+        self._cancel_delayed_start()
+        self._cancel_stop_offset()
+        
         if self.is_playing() or self.is_paused():
             prev_current = self._stop(**kwargs)
 
@@ -233,6 +296,7 @@ class ExailePlayer(object):
 
                 * `playback_player_pause`: indicates that the playback has been paused
         """
+        self._cancel_delayed_start()
         if self.is_playing():
             self._pause()
             event.log_event('playback_player_pause', self, self.current)
@@ -250,13 +314,14 @@ class ExailePlayer(object):
 
                 * `playback_player_resume`: indicates that the playback has been resumed
         """
+        self._cancel_delayed_start()
         if self.is_paused():
             self._unpause()
             event.log_event('playback_player_resume', self, self.current)
             return True
         return False
 
-    def _unpause():
+    def _unpause(self):
         raise NotImplementedError
 
     def toggle_pause(self):
@@ -285,9 +350,9 @@ class ExailePlayer(object):
 
     def get_position(self):
         """
-            Gets the current playback position
+            Gets the current playback position of the playing track
 
-            :returns: the position in milliseconds
+            :returns: the playback position in nanoseconds 
             :rtype: int
         """
         raise NotImplementedError
@@ -431,8 +496,8 @@ class ExailePlayer(object):
 
             value = [value]
 
-            if key == '__bitrate':
-                track.set_tag_raw('__bitrate', int(value[0]) / 1000)
+            if key == '__bitrate' or key == 'bitrate':
+                track.set_tag_raw('__bitrate', int(value[0]))
 
             # if there's a comment, but no album, set album to the comment
             elif key == 'comment' and not track.get_tag_raw('album'):
